"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.recordZodParser = exports.compileKeysEncoder = void 0;
const zod_1 = require("zod");
const utils_js_1 = require("../utils.js");
const schema_js_1 = require("./schema.js");
const utils_js_2 = require("./utils.js");
const withEncodedKeys = (schema, { transform }, zodSchema) => transform === false
    ? zodSchema
    : schema.keys.props.transform !== undefined
        ? zodSchema.transform((0, exports.compileKeysEncoder)(schema))
        : zodSchema;
const compileKeysEncoder = (schema) => (decoded) => {
    const encoded = {};
    for (const [key, value] of Object.entries(decoded)) {
        const encodedKey = schema.keys.props.transform.encode(key);
        encoded[encodedKey] = value;
    }
    return encoded;
};
exports.compileKeysEncoder = compileKeysEncoder;
const recordZodParser = (schema, options = {}) => {
    let zodFormatter;
    if (schema.keys.props.enum !== undefined && schema.props.partial !== true) {
        const elementsFormatter = (0, schema_js_1.schemaZodParser)(schema.elements, { ...options, defined: false });
        /**
         * @debt dependency "Using ZodObject until ZodStrictRecord is a thing: https://github.com/colinhacks/zod/issues/2623"
         */
        zodFormatter = withEncodedKeys(schema, options, zod_1.z.object(Object.fromEntries(schema.keys.props.enum.map(key => [key, elementsFormatter]))));
    }
    else {
        zodFormatter = zod_1.z.record((0, schema_js_1.schemaZodParser)(schema.keys, { ...options, defined: true }), (0, schema_js_1.schemaZodParser)(schema.elements, { ...options, defined: true }));
    }
    return (0, utils_js_2.withDefault)(schema, options, (0, utils_js_2.withOptional)(schema, options, (0, utils_js_1.withValidate)(schema, zodFormatter)));
};
exports.recordZodParser = recordZodParser;
