"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccessPattern = exports.IAccessPattern = void 0;
const dynamoDBToolboxError_js_1 = require("../../../errors/dynamoDBToolboxError.js");
const parser_js_1 = require("../../../schema/actions/parse/parser.js");
const queryCommand_js_1 = require("../../../table/actions/query/queryCommand.js");
const index_js_1 = require("../../../table/index.js");
const constants_js_1 = require("./constants.js");
class IAccessPattern extends index_js_1.TableAction {
    constructor(table, entities = [], schema, pattern, options = {}, meta = {}) {
        super(table, entities);
        this[constants_js_1.$schema] = schema;
        this[constants_js_1.$pattern] = pattern;
        this[constants_js_1.$options] = options;
        this[constants_js_1.$meta] = meta;
    }
    // IQueryCommand is needed for contravariance
    query(input) {
        const schema = this[constants_js_1.$schema];
        if (schema === undefined) {
            throw new dynamoDBToolboxError_js_1.DynamoDBToolboxError('actions.incompleteAction', {
                message: 'AccessPattern incomplete: Missing "schema" property'
            });
        }
        const pattern = this[constants_js_1.$pattern];
        if (pattern === undefined) {
            throw new dynamoDBToolboxError_js_1.DynamoDBToolboxError('actions.incompleteAction', {
                message: 'AccessPattern incomplete: Missing "pattern" property'
            });
        }
        const parser = new parser_js_1.Parser(schema);
        const transformedInput = parser.parse(input);
        const query = pattern(transformedInput);
        const options = this[constants_js_1.$options];
        return new queryCommand_js_1.QueryCommand(this.table, this[index_js_1.$entities], query, options);
    }
}
exports.IAccessPattern = IAccessPattern;
IAccessPattern.actionName = 'access-pattern';
class AccessPattern extends IAccessPattern {
    constructor(table, entities = [], schema, pattern, options = {}, meta = {}) {
        super(table, entities, schema, pattern, options, meta);
    }
    entities(...nextEntities) {
        return new AccessPattern(this.table, nextEntities, this[constants_js_1.$schema], this[constants_js_1.$pattern], this[constants_js_1.$options]);
    }
    schema(nextSchema) {
        return new AccessPattern(this.table, this[index_js_1.$entities], nextSchema, this[constants_js_1.$pattern], this[constants_js_1.$options]);
    }
    pattern(nextPattern) {
        return new AccessPattern(this.table, this[index_js_1.$entities], this[constants_js_1.$schema], nextPattern, this[constants_js_1.$options]);
    }
    options(nextOptions) {
        return new AccessPattern(this.table, this[index_js_1.$entities], this[constants_js_1.$schema], this[constants_js_1.$pattern], nextOptions);
    }
    meta(nextMeta) {
        return new AccessPattern(this.table, this[index_js_1.$entities], this[constants_js_1.$schema], this[constants_js_1.$pattern], this[constants_js_1.$options], nextMeta);
    }
}
exports.AccessPattern = AccessPattern;
