import { EntityAction } from '../../../entity/index.js';
import { DynamoDBToolboxError } from '../../../errors/dynamoDBToolboxError.js';
import { Parser } from '../../../schema/actions/parse/parser.js';
import { QueryCommand } from '../../../table/actions/query/queryCommand.js';
import { $meta, $options, $pattern, $schema } from './constants.js';
export class IAccessPattern extends EntityAction {
    constructor(entity, schema, pattern, options = {}, meta = {}) {
        super(entity);
        this[$schema] = schema;
        this[$pattern] = pattern;
        this[$options] = options;
        this[$meta] = meta;
    }
    // IQueryCommand is needed for contravariance
    query(input) {
        const schema = this[$schema];
        if (schema === undefined) {
            throw new DynamoDBToolboxError('actions.incompleteAction', {
                message: 'AccessPattern incomplete: Missing "schema" property'
            });
        }
        const pattern = this[$pattern];
        if (pattern === undefined) {
            throw new DynamoDBToolboxError('actions.incompleteAction', {
                message: 'AccessPattern incomplete: Missing "pattern" property'
            });
        }
        const parser = new Parser(schema);
        const transformedInput = parser.parse(input);
        const query = pattern(transformedInput);
        const options = this[$options];
        return new QueryCommand(this.entity.table, [this.entity], query, options);
    }
}
IAccessPattern.actionName = 'access-pattern';
export class AccessPattern extends IAccessPattern {
    constructor(entity, schema, pattern, options = {}, meta = {}) {
        super(entity, schema, pattern, options, meta);
    }
    schema(nextSchema) {
        return new AccessPattern(this.entity, nextSchema, this[$pattern], this[$options], this[$meta]);
    }
    pattern(nextPattern) {
        return new AccessPattern(this.entity, this[$schema], nextPattern, this[$options], this[$meta]);
    }
    options(nextOptions) {
        return new AccessPattern(this.entity, this[$schema], this[$pattern], nextOptions, this[$meta]);
    }
    meta(nextMeta) {
        return new AccessPattern(this.entity, this[$schema], this[$pattern], this[$options], nextMeta);
    }
}
