var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { GetCommand } from '@aws-sdk/lib-dynamodb';
import { EntityFormatter } from '../../../entity/actions/format/index.js';
import { $sentArgs } from '../../../entity/constants.js';
import { interceptable } from '../../../entity/decorator.js';
import { EntityAction } from '../../../entity/index.js';
import { DynamoDBToolboxError } from '../../../errors/index.js';
import { $key, $options } from './constants.js';
import { getItemParams } from './getItemParams/index.js';
export class GetItemCommand extends EntityAction {
    constructor(entity, key, options = {}) {
        super(entity);
        this[$key] = key;
        this[$options] = options;
    }
    key(nextKey) {
        return new GetItemCommand(this.entity, nextKey, this[$options]);
    }
    options(nextOptions) {
        return new GetItemCommand(this.entity, this[$key], nextOptions);
    }
    [$sentArgs]() {
        if (!this[$key]) {
            throw new DynamoDBToolboxError('actions.incompleteAction', {
                message: 'GetItemCommand incomplete: Missing "key" property'
            });
        }
        return [this[$key], this[$options]];
    }
    params() {
        return getItemParams(this.entity, ...this[$sentArgs]());
    }
    async send(documentClientOptions) {
        const getItemParams = this.params();
        const commandOutput = await this.entity.table
            .getDocumentClient()
            .send(new GetCommand(getItemParams), documentClientOptions);
        const { Item: item, ...restCommandOutput } = commandOutput;
        if (item === undefined) {
            return restCommandOutput;
        }
        const { attributes } = this[$options];
        const formattedItem = new EntityFormatter(this.entity).format(item, { attributes });
        return {
            Item: formattedItem,
            ...restCommandOutput
        };
    }
}
GetItemCommand.actionName = 'get';
__decorate([
    interceptable()
], GetItemCommand.prototype, "send", null);
