import { EntityParser } from '../../../entity/actions/parse/index.js';
import { DynamoDBToolboxError } from '../../../errors/index.js';
import { WriteTransaction } from '../transactWrite/transaction.js';
import { $key, $options } from './constants.js';
import { parseOptions } from './options.js';
export class DeleteTransaction extends WriteTransaction {
    constructor(entity, key, options = {}) {
        super(entity);
        this[$key] = key;
        this[$options] = options;
    }
    key(nextKey) {
        return new DeleteTransaction(this.entity, nextKey, this[$options]);
    }
    options(nextOptions) {
        return new DeleteTransaction(this.entity, this[$key], nextOptions);
    }
    params() {
        var _a;
        if (!this[$key]) {
            throw new DynamoDBToolboxError('actions.incompleteAction', {
                message: 'DeleteTransaction incomplete: Missing "key" property'
            });
        }
        const options = this[$options];
        const { key } = this.entity.build(EntityParser).parse(this[$key], { mode: 'key' });
        const awsOptions = parseOptions(this.entity, options);
        return {
            Delete: {
                TableName: (_a = options.tableName) !== null && _a !== void 0 ? _a : this.entity.table.getName(),
                Key: key,
                ...awsOptions
            }
        };
    }
}
DeleteTransaction.actionName = 'transactDelete';
