import { EntityConditionParser } from '../../../entity/actions/parseCondition/index.js';
import { rejectExtraOptions } from '../../../options/rejectExtraOptions.js';
import { parseReturnValuesOnConditionFalseOption } from '../../../options/returnValuesOnConditionFalse.js';
import { parseTableNameOption } from '../../../options/tableName.js';
import { isEmpty } from '../../../utils/isEmpty.js';
export const parseOptions = (entity, options) => {
    const transactionOptions = {};
    const { condition, returnValuesOnConditionFalse, tableName, ...extraOptions } = options;
    rejectExtraOptions(extraOptions);
    if (condition !== undefined) {
        const { ExpressionAttributeNames, ExpressionAttributeValues, ConditionExpression } = entity
            .build(EntityConditionParser)
            .parse(condition);
        if (!isEmpty(ExpressionAttributeNames)) {
            transactionOptions.ExpressionAttributeNames = ExpressionAttributeNames;
        }
        if (!isEmpty(ExpressionAttributeValues)) {
            transactionOptions.ExpressionAttributeValues = ExpressionAttributeValues;
        }
        transactionOptions.ConditionExpression = ConditionExpression;
    }
    if (returnValuesOnConditionFalse !== undefined) {
        transactionOptions.ReturnValuesOnConditionCheckFailure =
            parseReturnValuesOnConditionFalseOption(returnValuesOnConditionFalse);
    }
    if (tableName !== undefined) {
        // tableName is a meta-option, validated but not used here
        parseTableNameOption(tableName);
    }
    return transactionOptions;
};
