import { DynamoDBToolboxError } from '../../../errors/index.js';
import { cloneDeep } from '../../../utils/cloneDeep.js';
import { isObject } from '../../../utils/validation/isObject.js';
import { schemaParser } from './schema.js';
export function* itemParser(schema, inputValue, options = {}) {
    const { mode = 'put', fill = true, transform = true } = options;
    const parsers = {};
    let restEntries = [];
    const isInputValueObject = isObject(inputValue);
    if (isInputValueObject) {
        const additionalAttributeNames = new Set(Object.keys(inputValue));
        Object.entries(schema.attributes)
            .filter(([, attr]) => mode !== 'key' || attr.props.key)
            .forEach(([attrName, attr]) => {
            parsers[attrName] = schemaParser(attr, inputValue[attrName], {
                ...options,
                valuePath: [attrName],
                defined: false
            });
            additionalAttributeNames.delete(attrName);
        });
        restEntries = [...additionalAttributeNames.values()].map(attributeName => [
            attributeName,
            cloneDeep(inputValue[attributeName])
        ]);
    }
    if (fill) {
        if (isInputValueObject) {
            const defaultedValue = Object.fromEntries([
                ...Object.entries(parsers)
                    .map(([attrName, attr]) => [attrName, attr.next().value])
                    .filter(([, defaultedAttrValue]) => defaultedAttrValue !== undefined),
                ...restEntries
            ]);
            yield defaultedValue;
            const linkedValue = Object.fromEntries([
                ...Object.entries(parsers)
                    .map(([attrName, parser]) => [attrName, parser.next(defaultedValue).value])
                    .filter(([, linkedAttrValue]) => linkedAttrValue !== undefined),
                ...restEntries
            ]);
            yield linkedValue;
        }
        else {
            const defaultedValue = cloneDeep(inputValue);
            yield defaultedValue;
            const linkedValue = defaultedValue;
            yield linkedValue;
        }
    }
    if (!isInputValueObject) {
        throw new DynamoDBToolboxError('parsing.invalidItem', {
            message: 'Items should be objects',
            payload: {
                received: inputValue,
                expected: 'object'
            }
        });
    }
    const parsedValue = Object.fromEntries(Object.entries(parsers)
        .map(([attrName, attr]) => [attrName, attr.next().value])
        .filter(([, attrValue]) => attrValue !== undefined));
    if (transform) {
        yield parsedValue;
    }
    else {
        return parsedValue;
    }
    const transformedValue = Object.fromEntries(Object.entries(parsers)
        .map(([attrName, attr]) => {
        var _a;
        return [
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            (_a = schema.attributes[attrName].props.savedAs) !== null && _a !== void 0 ? _a : attrName,
            attr.next().value
        ];
    })
        .filter(([, attrValue]) => attrValue !== undefined));
    return transformedValue;
}
