import { z } from 'zod';
import { withValidate } from '../utils.js';
import { schemaZodFormatter } from './schema.js';
import { withOptional } from './utils.js';
export const anyOfZodFormatter = (schema, options = {}) => {
    let zodFormatter;
    const { discriminator } = schema.props;
    if (discriminator !== undefined) {
        // LIMITATION: Does not support nested `anyOf`s for now, should change with v4: https://v4.zod.dev/v4#upgraded-zdiscriminatedunion
        // LIMITATION: Does not support `savedAs` attributes for now as ZodEffects are not valid discriminatedUnion options
        zodFormatter = z.discriminatedUnion(discriminator, schema.elements.map(element => schemaZodFormatter(element, { ...options, defined: true })));
    }
    else {
        zodFormatter = z.union(schema.elements.map(element => schemaZodFormatter(element, { ...options, defined: true })));
    }
    return withOptional(schema, options, withValidate(schema, zodFormatter));
};
