import { DynamoDBToolboxError } from '../../errors/index.js';
import { checkSchemaProps } from '../utils/checkSchemaProps.js';
import { hasDefinedDefault } from '../utils/hasDefinedDefault.js';
export class ListSchema {
    constructor(elements, props) {
        this.type = 'list';
        this.elements = elements;
        this.props = props;
    }
    get checked() {
        return Object.isFrozen(this.props);
    }
    check(path) {
        if (this.checked) {
            return;
        }
        checkSchemaProps(this.props, path);
        const { required, hidden, savedAs } = this.elements.props;
        if (required !== undefined && required !== 'atLeastOnce' && required !== 'always') {
            throw new DynamoDBToolboxError('schema.list.optionalElements', {
                message: `Invalid list elements${path !== undefined ? ` at path '${path}'` : ''}: List elements must be required.`,
                path
            });
        }
        if (hidden !== undefined && hidden !== false) {
            throw new DynamoDBToolboxError('schema.list.hiddenElements', {
                message: `Invalid list elements${path !== undefined ? ` at path '${path}'` : ''}: List elements cannot be hidden.`,
                path
            });
        }
        if (savedAs !== undefined) {
            throw new DynamoDBToolboxError('schema.list.savedAsElements', {
                message: `Invalid list elements at path ${path !== undefined ? ` at path '${path}'` : ''}: List elements cannot be renamed (have savedAs prop).`,
                path
            });
        }
        if (hasDefinedDefault(this.elements)) {
            throw new DynamoDBToolboxError('schema.list.defaultedElements', {
                message: `Invalid list elements${path !== undefined ? ` at path '${path}'` : ''}: List elements cannot have default or linked values.`,
                path
            });
        }
        this.elements.check(`${path !== null && path !== void 0 ? path : ''}[n]`);
        Object.freeze(this.props);
        Object.freeze(this.elements);
    }
}
