import type { Entity } from '../../../entity/index.js';
import type { InputValue, Schema, TransformedValue } from '../../../schema/index.js';
import type { IQueryCommand, Query, QueryOptions } from '../../../table/actions/query/index.js';
import { QueryCommand } from '../../../table/actions/query/queryCommand.js';
import type { Table } from '../../../table/index.js';
import { TableAction } from '../../../table/index.js';
import { $meta, $options, $pattern, $schema } from './constants.js';
interface AccessPatternMetadata {
    title?: string;
    description?: string;
    [x: string]: unknown;
}
export declare class IAccessPattern<TABLE extends Table = Table, ENTITIES extends Entity[] = Entity[], SCHEMA extends Schema = Schema, QUERY extends Query<TABLE> = Query<TABLE>, OPTIONS extends QueryOptions<TABLE, ENTITIES> = QueryOptions<TABLE, ENTITIES>> extends TableAction<TABLE, ENTITIES> {
    static actionName: "access-pattern";
    [$schema]?: SCHEMA;
    [$pattern]?: (input: Schema extends SCHEMA ? any : TransformedValue<SCHEMA>) => QUERY;
    [$options]: OPTIONS;
    [$meta]: AccessPatternMetadata;
    constructor(table: TABLE, entities?: ENTITIES, schema?: SCHEMA, pattern?: (input: TransformedValue<SCHEMA>) => QUERY, options?: OPTIONS, meta?: AccessPatternMetadata);
    query(input: InputValue<SCHEMA>): Table extends TABLE ? IQueryCommand : QueryCommand<TABLE, ENTITIES, Query<TABLE>, OPTIONS>;
}
export declare class AccessPattern<TABLE extends Table = Table, ENTITIES extends Entity[] = Entity[], SCHEMA extends Schema = Schema, QUERY extends Query<TABLE> = Query<TABLE>, OPTIONS extends QueryOptions<TABLE, ENTITIES> = QueryOptions<TABLE, ENTITIES>> extends IAccessPattern<TABLE, ENTITIES, SCHEMA, QUERY, OPTIONS> {
    constructor(table: TABLE, entities?: ENTITIES, schema?: SCHEMA, pattern?: (input: TransformedValue<SCHEMA>) => QUERY, options?: OPTIONS, meta?: AccessPatternMetadata);
    entities<NEXT_ENTITIES extends Entity[]>(...nextEntities: NEXT_ENTITIES): AccessPattern<TABLE, NEXT_ENTITIES, SCHEMA, QUERY, OPTIONS extends QueryOptions<TABLE, NEXT_ENTITIES> ? OPTIONS : QueryOptions<TABLE, NEXT_ENTITIES>>;
    schema<NEXT_SCHEMA extends Schema>(nextSchema: NEXT_SCHEMA): AccessPattern<TABLE, ENTITIES, NEXT_SCHEMA, QUERY, OPTIONS>;
    pattern<NEXT_QUERY extends Query<TABLE>>(nextPattern: (input: TransformedValue<SCHEMA>) => NEXT_QUERY): AccessPattern<TABLE, ENTITIES, SCHEMA, NEXT_QUERY, OPTIONS>;
    options<NEXT_OPTIONS extends QueryOptions<TABLE, ENTITIES>>(nextOptions: NEXT_OPTIONS): AccessPattern<TABLE, ENTITIES, SCHEMA, QUERY, NEXT_OPTIONS>;
    meta(nextMeta: AccessPatternMetadata): AccessPattern<TABLE, ENTITIES, SCHEMA, QUERY, OPTIONS>;
}
export {};
