import { DynamoDBToolboxError } from '../../../errors/dynamoDBToolboxError.js';
import { Parser } from '../../../schema/actions/parse/parser.js';
import { QueryCommand } from '../../../table/actions/query/queryCommand.js';
import { $entities, TableAction } from '../../../table/index.js';
import { $meta, $options, $pattern, $schema } from './constants.js';
export class IAccessPattern extends TableAction {
    constructor(table, entities = [], schema, pattern, options = {}, meta = {}) {
        super(table, entities);
        this[$schema] = schema;
        this[$pattern] = pattern;
        this[$options] = options;
        this[$meta] = meta;
    }
    // IQueryCommand is needed for contravariance
    query(input) {
        const schema = this[$schema];
        if (schema === undefined) {
            throw new DynamoDBToolboxError('actions.incompleteAction', {
                message: 'AccessPattern incomplete: Missing "schema" property'
            });
        }
        const pattern = this[$pattern];
        if (pattern === undefined) {
            throw new DynamoDBToolboxError('actions.incompleteAction', {
                message: 'AccessPattern incomplete: Missing "pattern" property'
            });
        }
        const parser = new Parser(schema);
        const transformedInput = parser.parse(input);
        const query = pattern(transformedInput);
        const options = this[$options];
        return new QueryCommand(this.table, this[$entities], query, options);
    }
}
IAccessPattern.actionName = 'access-pattern';
export class AccessPattern extends IAccessPattern {
    constructor(table, entities = [], schema, pattern, options = {}, meta = {}) {
        super(table, entities, schema, pattern, options, meta);
    }
    entities(...nextEntities) {
        return new AccessPattern(this.table, nextEntities, this[$schema], this[$pattern], this[$options]);
    }
    schema(nextSchema) {
        return new AccessPattern(this.table, this[$entities], nextSchema, this[$pattern], this[$options]);
    }
    pattern(nextPattern) {
        return new AccessPattern(this.table, this[$entities], this[$schema], nextPattern, this[$options]);
    }
    options(nextOptions) {
        return new AccessPattern(this.table, this[$entities], this[$schema], this[$pattern], nextOptions);
    }
    meta(nextMeta) {
        return new AccessPattern(this.table, this[$entities], this[$schema], this[$pattern], this[$options], nextMeta);
    }
}
